/* This file is part of the xFlows library.
 * Copyright (C) 2009 Jonas Latt
 * E-mail contact: jonas@lbmethod.org
 * The most recent release of xFlows can be downloaded at 
 * <http://www.lbmethod.org/xflows/>
 *
 * The library xFlows is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * The library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/* Orestis Malaspinas contributed this code.
 */

/** \file
 * Descriptor for all types of 2D and 3D lattices. In principle, thanks
 * to the fact that the OpenLB code is generic, it is sufficient to
 * write a new descriptor when a new type of lattice is to be used.
 *  -- header file
 */
#ifndef ADV_DIFF_TURBULENT_VISCOSITY_LATTICES_H
#define ADV_DIFF_TURBULENT_VISCOSITY_LATTICES_H

#include "core/globalDefs.h"
#include <vector>
#include "latticeBoltzmann/externalFields.h"
#include "latticeBoltzmann/advectionDiffusionLattices.h"

namespace plb {

/// Descriptors for the 2D and 3D lattices.

namespace descriptors 
{
//===========================================================================//
//=================== AdvDiffusionMelting Lattice Descriptors=================//
//===========================================================================//
    
    

    struct Vel3dTubulentDiffusionDescriptor {
        static const int numScalars = 6;
        static const int numSpecies = 4;
        static const int velocityBeginsAt = 0;
        static const int sizeOfVelocity   = 3;
	static const int turbulentViscosityBeginsAt  = 3;// liquid Fraction at previous Time step (n-1)
        static const int sizeOfturbulentViscosity   = 1;
	static const int turbulentViscosityBeginsAt  = 4;// liquid Fraction at previous Time step (n-1)
        static const int sizeOfturbulentViscosity   = 1;
	static const int turbulentViscosityBeginsAt  = 5;// liquid Fraction at previous Time step (n-1)
        static const int sizeOfturbulentViscosity   = 1;
		
    };
    
    struct Vel3dTurubulentViscosityAdvectionDiffusionBase {
        typedef Vel3dTubulentDiffusionDescriptor ExternalField;
    };
    
    
    
	/// AD MELTING D3Q7 lattice
    template <typename T> struct AdvDiff_withTurbulentViscosity_D3Q7Descriptor 
    : public D3Q7DescriptorBase<T>, public Vel3dTurubulentViscosityAdvectionDiffusionBase
    {  
	static const char name[];    
    };
 
    
    
    
    template <typename T>
    const char 
    AdvDiff_withTurbulentViscosity_D3Q7Descriptor<T>::name[] = 
		"AdvDiff_TurbulentViscosity";
    

}  // namespace descriptors

}  // namespace plb

#endif
